/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.mail.service.impl.guests;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import org.json.JSONArray;
import org.json.JSONObject;
import com.openexchange.exception.OXException;
import com.openexchange.guard.api.GuardApis;
import com.openexchange.guard.exceptions.GuardExceptionCodes;
import com.openexchange.guard.internal.AbstractGuardAccess;
import com.openexchange.guard.internal.GuardApiImpl;
import com.openexchange.guard.internal.authentication.GuardAuthenticationToken;
import com.openexchange.session.Session;

/**
 * {@link GuardGuestMailDeleter} deletes OX Guard guest E-Mails
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v7.10.0
 */
public class GuardGuestMailDeleter extends AbstractGuardAccess {

    /**
     * Internal method to create a delete object
     *
     * @param mailIds The IDs of the guest E-Mails to delete
     * @return The created delete object ready to include in the request
     */
    private JSONObject createDelete(String[] mailIds, String folderId) {
        JSONObject jsonObject = new JSONObject();
        if(mailIds != null) {
            jsonObject.putSafe("mailIds", new JSONArray(Arrays.asList(mailIds)));
        }
        if(folderId != null) {
            jsonObject.putSafe("folderId", folderId);
        }
        return jsonObject;
    }

    private void deleteMessagesInternal(Session session, GuardAuthenticationToken authToken, String[] mailIds, String folderId) throws OXException {
        final GuardApiImpl guardGuestApi = requireGuardApi(GUEST_ENDPOINT);
        final JSONObject authJson = GuardGuestAuthHelper.toAuthJSON(authToken, session);
        final JSONObject json = new JSONObject()
            .putSafe("auth", authJson)
            .putSafe("delete", createDelete(mailIds, folderId));
        ByteArrayInputStream jsonStream = new ByteArrayInputStream(json.toString().getBytes(StandardCharsets.UTF_8));

        try (InputStream result = guardGuestApi.processResource(GuardApis.mapFor("action", "deletemessages"),
            jsonStream,
            null,
            "json",
            session)) {
            //no-op
        } catch (IOException e) {
            throw GuardExceptionCodes.UNKNOWN_ERROR.create(e, e.getMessage());
        }

    }

    /**
     * Deletes a set of mail items
     *
     * @param session The session
     * @param authToken The Guard authentication token
     * @param mailIds The IDs of the mail itesm to delete
     * @throws OXException
     */
    public void deleteMessages(Session session, GuardAuthenticationToken authToken, String[] mailIds) throws OXException {
        final String folderId = null; //Not specifying the folderID, because we want to delete a bunch of mails by ID
        deleteMessagesInternal(session, authToken, mailIds, folderId);
    }

    /**
     * Deletes all mail items in a given folder
     *
     * @param session The sesion
     * @param authToken The Guard authentication token
     * @param folderId The ID of the folder to delete all items from
     * @throws OXException
     */
    public void deleteAllMessages(Session session, GuardAuthenticationToken authToken, String folderId) throws OXException {
        final String[] mailIds = null; //Not specifying the mail IDs, because we want to delete all mails from a specific folder
        deleteMessagesInternal(session, authToken, mailIds, folderId);
    }
}
